<?php

namespace BackOfficeOptimizer\Libraries;

/**
 * An static class
 */
class JWT {
 
    const SECRET_KEY = "PCIHRE0CCL4gCf9YTjEEpuDvF8G";
    const expirationTime = 24 * 60; //in minutes

    public static function generateToken($payload) {
        $header= array(
            "typ" => "JWT",
            "exp" => time() + self::expirationTime * 60
        );
        $unsignedToken = self::base64_url_encode(json_encode((object) $header)) . "." . self::base64_url_encode(json_encode((object) $payload));
        return $unsignedToken . "." . self::sign($unsignedToken);
    }

    public static function validateToken($token) {
        try {
            $sections  = explode(".", $token);
            $content   = $sections[0] . "." . $sections[1];
            $signature = $sections[2];
            $header = self::getTokenHeader($token);
            if (self::sign($content) != $signature) {
                throw new \Exception();
            } else if(!isset($header["exp"]) || time() > $header["exp"]){
                throw new \Exception("expired");
            }
            return true;
        }
        catch (\Exception $e) {
            self::unsetSessionCookie('optimizer');
            return false;
        }
    }

    public static function setSessionCookie($name, $value = "", $time = 0, $path = "/", $domain = "", $secure = false, $httponly = false) {
        setcookie($name, $value, $time, $path, $domain, $secure, $httponly);
    }

    public static function unsetSessionCookie($name) {
        self::setSessionCookie($name);
    }

    public static function getTokenData($token) {
        $segments = explode('.', $token);
        list($headb64, $bodyb64, $cryptob64) = $segments;
        return json_decode(self::base64_url_decode($bodyb64), true);
    }

    public static function getTokenHeader($token) {
        $segments = explode('.', $token);
        list($headb64, $bodyb64, $cryptob64) = $segments;
        return json_decode(self::base64_url_decode($headb64), true);
    }

    private static function sign($data) {
        return self::base64_url_encode(hash_hmac("sha256", $data, self::SECRET_KEY, true));
    }

    private static function base64_url_encode($val) {
        return strtr(base64_encode($val), '+/=', '-_,');
    }

    private static function base64_url_decode($val) {
        return base64_decode(strtr($val, '-_,', '+/='));
    }
}